<?php

namespace ByDedi\Module\CustomForm\Model;

use Boutique;
use Form\Constraint\NotBlank;
use LogicException;
use Model;
use Form\Constraint\Constraint;
use Query;

/**
 * Class FormField
 *
 * @property int $form_field_id
 * @property int $form_field_type_id
 */
class FormField extends Model
{
    const CONSTRAINT_NOT_BLANK_ID = 1;
    const CONSTRAINT_EMAIL_ID = 2;
    const CONSTRAINT_FRENCH_PHONE_ID = 3;

    const TYPE_TEXT_ID = 1;
    const TYPE_SELECT_ID = 2;
    const TYPE_RADIO_ID = 3;
    const TYPE_CHECKBOX_ID = 4;

    const TYPE_WIHT_OPTIONS = [
        self::TYPE_TEXT_ID => false,
        self::TYPE_SELECT_ID => true,
        self::TYPE_RADIO_ID => true,
        self::TYPE_CHECKBOX_ID => false,
    ];

    protected $table = "form_field";

    /**
     * @var Constraint[]
     */
    protected $constraints;

    /**
     * @var array[]
     */
    protected $options;

    /**
     * @var string
     */
    protected $name = '';

    /**
     * @param $formFieldTypeId
     * @return bool
     */
    public static function typeHasOption($formFieldTypeId)
    {
        return !empty(self::TYPE_WIHT_OPTIONS[$formFieldTypeId]);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function get($formvars = array())
    {
        $q = new Query();
        $boutique = Boutique::getInstance();
        $q->select('*')
            ->from('form_field')
            ->leftJoin('form_field_nom', "AND", ['ffn.form_field_id', 'ff.form_field_id'], ['ffn.langue_id', $boutique->getLangue()->id])
            ->innerJoin('form_field_boutique', 'AND',
                ['ffb.form_field_id', 'ff.form_field_id'],
                ['ffb.boutique_id', $boutique->id],
                ['ffb.form_field_boutique_statut_id', 1]
            )
            ->where('ff.form_field_statut_id', 1)
            ->order('ff.form_field_rang')
        ;

        if (!empty($formvars['form_id'])) {
            $q->where('ff.form_id', $formvars['form_id']);
        }

        return $this->sql->getAll($q->build());
    }

    public function fetchConstraints()
    {
        if (empty($this->getFormFieldId())) {
            throw new LogicException('Trying to get Constraints from an unloaded FormField');
        }

        $q = new Query();
        $boutique = Boutique::getInstance();
        $q->select('*')
            ->from('form_field_constraint')
            ->innerJoin('form_field_constraint_boutique', 'AND',
                ['ffcb.form_field_constraint_id', 'ffc.form_field_constraint_id'],
                ['ffcb.boutique_id', $boutique->id],
                ['ffcb.form_field_constraint_boutique_statut_id', 1]
            )
        ;

        $q->where('ffc.form_field_id', $this->getFormFieldId());

        $arrConstraints = $this->sql->getAll($q->build());
        return $arrConstraints;
    }

    /**
     * @return int
     */
    public function getFormFieldId()
    {
        return $this->form_field_id;
    }

    /**
     * @param int $form_field_id
     * @return FormField
     */
    public function setFormFieldId($form_field_id)
    {
        $this->form_field_id = $form_field_id;
        return $this;
    }

    /**
     * @return Constraint[]
     */
    public function getConstraints()
    {
        return $this->constraints;
    }

    /**
     * @param Constraint[] $constraints
     * @return FormField
     */
    public function setConstraints($constraints)
    {
        $this->constraints = $constraints;
        return $this;
    }

    /**
     * @return array[]
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param array[] $options
     * @return FormField
     */
    public function setOptions($options)
    {
        $this->options = $options;
        return $this;
    }

    /**
     * @return int
     */
    public function getFormFieldTypeId()
    {
        return $this->form_field_type_id;
    }

    /**
     * @param int $form_field_type_id
     * @return FormField
     */
    public function setFormFieldTypeId($form_field_type_id)
    {
        $this->form_field_type_id = $form_field_type_id;
        return $this;
    }

    public function hasOptions()
    {
        $formFieldTypeId = $this->getFormFieldTypeId();
        return self::typeHasOption($formFieldTypeId);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function fetchOptions($formvars = [])
    {
        if (!isset($formvars['form_field_id'])) {
            $formvars['form_field_id'] = $this->getFormFieldId();
        }
        $boutique = Boutique::getInstance();

        $q = new Query();
        $q->select('*')
            ->from('form_field_option')
            ->leftJoin(
                'form_field_option_nom',
                "AND",
                ['ffon.form_field_option_id', 'ffo.form_field_option_id'],
                ['ffon.langue_id', $boutique->getLangue()->id]
            )
            ->order('ffo.form_field_option_rang')
        ;

        if (!empty($formvars['form_field_id'])) {
            $q->where('ffo.form_field_id', $formvars['form_field_id']);
        }

        if (!empty($formvars['form_field_option_value'])) {
            $q->where('ffo.form_field_option_value', $formvars['form_field_option_value']);
        }

        $options = $this->sql->getAll($q->build());
        return $options;
    }

    /**
     * @param string $value
     * @return array
     */
    public function getOptionByValue($value)
    {
        $options = $this->fetchOptions([
            'form_field_option_value' => $value
        ]);
        if (empty($options)) {
            return [];
        }
        return $options[0];
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return FormField
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    public function shouldBeFilled()
    {
        foreach ($this->getConstraints() as $constraint) {
            if ($constraint instanceof NotBlank) {
                return true;
            }
        }
        return false;
    }
}